Package: ADAM
Type: Package
Title: ADAM: Activity and Diversity Analysis Module
Version: 1.29.0
Date: 2023-04-10
Authors@R: 
    c(person(given = "André Luiz",
                 family = "Molan",
                 role = "aut",
                 email = "andre.molan@unesp.br"),
      person(given = "Giordano Bruno",
                 family = "Sanches Seco",
                 role = "ctb"),
      person(given = "Agnes",
                family = "Takeda",
                role = "ctb"),
      person(given = "Jose",
                family = "Rybarczyk Filho",
                role = c("ctb", "cre", "ths"),
                email = "jose.luiz@unesp.br")
			 )
Maintainer: Jose Luiz Rybarczyk Filho <jose.luiz@unesp.br>
Description: ADAM is a GSEA R package created to group a set of genes from
    comparative samples (control versus experiment) belonging to different
    species according to their respective functions (Gene Ontology and KEGG
    pathways as default) and show their significance by calculating p-values
    referring togene diversity and activity. Each group of genes is called GFAG
    (Group of Functionally Associated Genes). 
Depends: R(>= 3.5), stats, utils, methods
Imports: Rcpp (>= 0.12.18), GO.db (>= 3.6.0), KEGGREST (>= 1.20.2),
        knitr, pbapply (>= 1.3-4), dplyr (>= 0.7.6), DT (>= 0.4),
        stringr (>= 1.3.1), SummarizedExperiment (>= 1.10.1)
Suggests: testthat, rmarkdown, BiocStyle
LinkingTo: Rcpp
SystemRequirements: C++11
Encoding: UTF-8
RoxygenNote: 6.1.0
License: GPL (>= 2)
biocViews: GeneSetEnrichment, Pathways, KEGG, GeneExpression,
        Microarray
VignetteBuilder: knitr
LazyLoad: yes
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/ADAM
git_branch: devel
git_last_commit: 8f82406
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:10:30 UTC; biocbuild
Author: André Luiz Molan [aut],
  Giordano Bruno Sanches Seco [ctb],
  Agnes Takeda [ctb],
  Jose Rybarczyk Filho [ctb, cre, ths]

# Generated by roxygen2: do not edit by hand

export(ADAnalysis)
export(GFAGAnalysis)
import(GO.db)
import(KEGGREST)
import(Rcpp)
import(knitr)
import(methods)
import(pbapply)
importFrom(DT,datatable)
importFrom(Rcpp,evalCpp)
importFrom(SummarizedExperiment,assay)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(stats,fisher.test)
importFrom(stats,p.adjust)
importFrom(stats,wilcox.test)
importFrom(stringr,str_split)
importFrom(utils,installed.packages)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(ADAM)

# Adam/Moses Bot Framework

A declarative framework for building IRC bots based on POE and Moose.

## Description

The Adam/Moses Bot Framework provides a simple, declarative way to create IRC bots in Perl. It's built on top of POE::Component::IRC and uses Moose for clean object-oriented design.

Moses provides declarative sugar that makes bot creation straightforward, while Adam is the underlying implementation class.

## Installation

From CPAN:

```bash
cpanm Adam
```

From source:

```bash
git clone https://github.com/perigrin/adam-bot-framework.git
cd adam-bot-framework
cpanm --installdeps .
```

## Basic Usage

```perl
package MyBot;
use Moses;
use namespace::autoclean;

server 'irc.perl.org';
nickname 'mybot';
channels '#bots';

event irc_bot_addressed => sub {
    my ($self, $nickstr, $channel, $msg) = @_[OBJECT, ARG0, ARG1, ARG2];
    my ($nick) = split /!/, $nickstr;
    $self->privmsg($channel => "$nick: Hello!");
};

__PACKAGE__->run unless caller;
```

## Features

- **Declarative syntax** - Simple DSL for bot configuration
- **Plugin system** - Easy to extend with POE::Component::IRC plugins
- **Multiple event loops** - Supports both POE (default) and IO::Async
- **Command-line options** - Built-in support via MooseX::Getopt
- **Logging** - Configurable logging via MooseX::LogDispatch

## IO::Async Support

You can use IO::Async as the event loop (requires IO::Async::Loop::POE):

```perl
package AsyncBot;
use Moses;
use namespace::autoclean;

server 'irc.perl.org';
nickname 'asyncbot';
channels '#ai';

event irc_join => sub {
    my ( $self, $nickstr, $channel ) = @_[ OBJECT, ARG0, ARG1 ];
    my ($nick) = split /!/, $nickstr;
    return unless $nick eq $self->get_nickname;
    $self->privmsg( $channel => "Hello, artificial humans!" );
};

__PACKAGE__->async unless caller;
```

Use `$bot->stop` to cleanly stop the event loop in both POE and IO::Async modes.

## Links

- [CPAN](https://metacpan.org/pod/Adam)
- [GitHub](https://github.com/perigrin/adam-bot-framework)
- [Documentation](https://metacpan.org/pod/Moses)

This archive contains the distribution Adam,
version 1.003:

  The patriarch of IRC Bots

This software is copyright (c) 2010 by Chris Prather, Torsten Raudssus.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.


This README file was generated by Dist::Zilla::Plugin::Readme v6.037.

########
## Function to run diversity and activity analysis
########

makeAnalysis <- function(ComparisonID,ECGObject,completeTest){
    ###
    #Building the comparison dataframe from expression data
    ###
    compID <- unlist(strsplit(ComparisonID,","))

    message(paste0("Analysis: ",compID[1],"_Vs_",compID[2]))

    Comparison <- data.frame(ECGObject@ExpressionData$gene,
                            ECGObject@ExpressionData[,grep(compID[1],
                            colnames(ECGObject@ExpressionData))],
                            ECGObject@ExpressionData[,grep(compID[2],
                            colnames(ECGObject@ExpressionData))])
    colnames(Comparison) <- c("gene","control","experiment")

    ###
    #Building group of functionally associated genes (GFAGs)
    ###
    message("Building GFAGs ...")

    pboptions(style = 1, char = ">")

    GroupComparisons <- pblapply(ECGObject@DBSpeciesFunctionsSample,
                                function(DBFSample,Comparison){
                                Comparison[Comparison$gene %in% DBFSample,]},
                                Comparison=Comparison)
    GroupComparisons <- GroupComparisons[lapply(lapply(GroupComparisons,
                            function(GComparison)
                            {if(nrow(GComparison)>=ECGObject@MinGene & 
                            nrow(GComparison)<=ECGObject@MaxGene){GComparison}
                            else{NULL}}),length)>0]
    GroupComparisons <- GroupComparisons[order(names(GroupComparisons))]

    ###
    #Calculating gene activity and diversity
    ###
    message("Calculating activity and diversity ...")

    GroupStatistics <- pblapply(GroupComparisons, FUN = SampleStatistics,
                                Control = compID[1], Experiment = compID[2])

    if(!completeTest){
        GroupStatistics<-as.data.frame(do.call(rbind, lapply(GroupStatistics,
                        function(GroupStats){as.vector(unlist(GroupStats))})))
        GroupStatistics$ID <- vapply(rownames(GroupStatistics),
                        function(GroupStats) 
                        trimws(unlist(strsplit(GroupStats,"<==>"))[1]), 
                        FUN.VALUE = character(1))
        GroupStatistics$Description <- vapply(rownames(GroupStatistics),
                                        function(GroupStats) trimws(unlist(
                                        strsplit(GroupStats,"<==>")))[2], 
                                        FUN.VALUE = character(1))
        
        
        GroupStatistics$Raw_Number_Genes<-vapply(GroupStatistics$ID,
                                        function(GroupStats) 
                                        ECGObject@DBSpeciesFunctionsRaw[[
                                        grep(GroupStats,names(
                                        ECGObject@DBSpeciesFunctionsRaw))]], 
                                        FUN.VALUE = numeric(1))
        
        
        rownames(GroupStatistics) <- NULL
        GroupStatistics <- GroupStatistics[,c(8,9,10,1:7)]
        colnames(GroupStatistics) <- c("ID","Description","Raw_Number_Genes",
                                    "Sample_Number_Genes", 
                                    paste0("H_",compID[1]),
                                    paste0("H_",compID[2]),
                                    paste0("N_",compID[1]),
                                    paste0("N_",compID[2]),"h","n")

        ResultAnalysis <- GroupStatistics
    }else{
        ###
        #Determining the minimum and maximum number of genes for
        #each function group
        ###
        GroupSize <- sort(unique(vapply(GroupComparisons,
                    function(GComparison) nrow(GComparison),
                    FUN.VALUE = numeric(1))))
        
        ###
        #Filtering GFAGs by size
        ###
        message("Filtering GFAGs by size ...")
        
        ResultBootstrap<-pblapply(GroupSize, function(GSize,GroupStatistics){
                            GroupStatistics[lapply(lapply(GroupStatistics,
                            function(GStats,GSizeStats){
                            if(GStats[1,1]==GSizeStats){GStats}else{NULL}}, 
                            GSizeStats = GSize),length)>0]},
                            GroupStatistics=GroupStatistics)
        
        ###
        #Running Bootstrap
        ###
        message("Running bootstrap. This may take a few minutes ...")
        
        ComparisonExpression<-data.frame(Comparison$control,
                                        Comparison$experiment)
        colnames(ComparisonExpression) <- c("control","experiment")
        
        ResultBootstrap <- pblapply(ResultBootstrap, FUN = PrepareBootstrap,
                                    ComparisonExpression=ComparisonExpression,
                                    ECGObject = ECGObject)

        ResultBootstrap <- as.data.frame(do.call(rbind,
                                lapply(ResultBootstrap,function(ResultBtp){
                                do.call(rbind,lapply(ResultBtp,
                                        function(ResultBtpBind)
                                        as.vector(unlist(ResultBtpBind))))})))
        
        ###
        #Manipulating Bootstrap output
        ###
        colnames(ResultBootstrap) <- c("Sample_Number_Genes", 
                                    paste0("H_",compID[1]),
                                    paste0("H_",compID[2]),
                                    paste0("N_",compID[1]),
                                    paste0("N_",compID[2]),"h","n",
                                    "pValue_h","pValue_n")
        ResultBootstrap$ID <- vapply(rownames(ResultBootstrap),
                                function(ResultBtp)
                                trimws(unlist(strsplit(ResultBtp,"<==>"))[1]),
                                FUN.VALUE = character(1))
        ResultBootstrap$Description <- vapply(rownames(ResultBootstrap), 
                                            function(ResultBtp)
                                            trimws(unlist(strsplit(ResultBtp,
                                            "<==>")))[2],
                                            FUN.VALUE=character(1))
        rownames(ResultBootstrap) <- NULL
        ResultBootstrap <- ResultBootstrap[,c(length(ResultBootstrap)-1,
                                            length(ResultBootstrap),
                                            1:(length(ResultBootstrap)-2))]
        
        ###
        #Correcting p-values
        ###
        message(paste0("Correcting bootstrap p-values by ",
                    ECGObject@PCorrectionMethod," ..."))
        
        ResultBootstrap$qValue_h <- p.adjust(ResultBootstrap$pValue_h,
                                        method=ECGObject@PCorrectionMethod)
        ResultBootstrap$qValue_n <- p.adjust(ResultBootstrap$pValue_n,
                                        method=ECGObject@PCorrectionMethod)
        ResultBootstrap$Significance_h <- vapply(ResultBootstrap$qValue_h,
                                function(ResultQValue)
                                ifelse(ResultQValue<=ECGObject@PCorrection,
                                    "significative","not_significative"),
                                FUN.VALUE = character(1))
        ResultBootstrap$Significance_n <- vapply(ResultBootstrap$qValue_n,
                                function(ResultQValue)
                                ifelse(ResultQValue<=ECGObject@PCorrection,
                                    "significative","not_significative"), 
                                FUN.VALUE = character(1))
        
        ###
        #Adding the raw gene numbers per function to the result dataframe
        ###
        ResultBootstrap$Raw_Number_Genes <- vapply(ResultBootstrap$ID,
                                        function(GroupID)
                                        ECGObject@DBSpeciesFunctionsRaw[[
                                        grep(GroupID,names(
                                        ECGObject@DBSpeciesFunctionsRaw))]],
                                        FUN.VALUE = numeric(1))
        
        ###
        #Changing the order of the dataframe result columns
        ###
        ResultBootstrap <- ResultBootstrap[,c(1,2,length(ResultBootstrap),
                                            3:(length(ResultBootstrap)-1))]
        ResultBootstrap <- arrange(ResultBootstrap,ResultBootstrap[,1])
        
        ###
        #Wicoxon Test
        ###
        if(ECGObject@WilcoxonTest){
            message("Runing Wilcox rank sum test ...")
            ResultBootstrap$Wilcox_pvalue <- vapply(GroupComparisons,
                                            FUN = WCAnalysis, 
                                            FUN.VALUE = numeric(1))
            ResultBootstrap$Wilcox_qvalue <- p.adjust(as.numeric(
                                        ResultBootstrap$Wilcox_pvalue),
                                        method=ECGObject@PCorrectionMethod)
            ResultBootstrap$Wilcox_significance<-ifelse(
                                            ResultBootstrap$Wilcox_qvalue <=
                                            ECGObject@PCorrection,
                                            "significative",
                                            "not significative")
        }
        
        ###
        #Fisher Test
        ###
        if(ECGObject@FisherTest){
            message("Running Fisher exact test ...")
            ResultBootstrap$Fisher_pvalue <- vapply(GroupComparisons,
                                                    FUN=FSHAnalysis,
                                                    Comparison = Comparison, 
                                                    FUN.VALUE = numeric(1))
            ResultBootstrap$Fisher_qvalue <- p.adjust(
                                        as.numeric(
                                        ResultBootstrap$Fisher_pvalue),
                                        method=ECGObject@PCorrectionMethod)
            ResultBootstrap$Fisher_significance <- ifelse(
                                            ResultBootstrap$Fisher_qvalue <= 
                                            ECGObject@PCorrection,
                                            "significative",
                                            "not significative")
        }
        
        ResultAnalysis <- ResultBootstrap
    }

    ###
    #Analysis concluded
    ###
    message(paste0("Analysis of ",compID[1],"_Vs_",compID[2]),
            " succesfully concluded!")
    message("")

    return(ResultAnalysis)
}

########
# Function to calculate the activity of gene set.
########
Activity <- function(DataGroup, ColumnDataNumber){
    # N = gene activity of a GFAG
    N <- sum(DataGroup[,ColumnDataNumber])
    return(N)
}

########
# Function to calculate the diversity of gene set.
########
Diversity <- function(DataGroup, ColumnDataNumber){
    ColumnData <- DataGroup[,ColumnDataNumber]
    ColumnData <- ColumnData[ColumnData>0]
    # H = gene diversity of a GFAG
    if(length(ColumnData)>0 & length(DataGroup[,ColumnDataNumber])>1){
        H <- -(1/(log(length(DataGroup[,ColumnDataNumber]),
                base = exp(1))))*sum(ColumnData/(sum(ColumnData))*
                log(ColumnData/sum(ColumnData), base = exp(1)), na.rm = FALSE)
    }else{
        H<-0
    }
    return(H)
}

########
# Function to calculate activity (N), diversity (H), relative activity (n) and  
# relative diversity (h) of a gene set.
########
SampleStatistics <- function(GroupComparisons,Control,Experiment){
    TempDataFrame <- GroupComparisons[,c(2,3)]
    H_Control <- Diversity(TempDataFrame, 1)
    H_Experiment <- Diversity(TempDataFrame, 2)
    N_Control <- Activity(TempDataFrame, 1)
    N_Experiment <- Activity(TempDataFrame, 2)
    if((H_Experiment + H_Control)>0){
        # h = gene relative diversity between control and experiment samples
        h <- H_Experiment/(H_Experiment + H_Control)
    }else{
        h <- 0
    }
    if((N_Experiment + N_Control)>0){
        # n = gene relative activity between control and experiment samples
        n <- N_Experiment/(N_Experiment + N_Control)
    }else{
        n <-0
    }
    Result<-data.frame(nrow(GroupComparisons),H_Control,H_Experiment,
                        N_Control, N_Experiment, h, n)
    colnames(Result) <- c("Sample_Number_Genes", paste0("H_",Control),
                        paste0("H_",Experiment),
                        paste0("N_",Control), paste0("N_",Experiment),
                        "h","n")
    return(Result)
}

########
## Function to prepare data expression function groups for bootstrapping
########
PrepareBootstrap <- function(GroupFunction,ComparisonExpression,ECGObject){

    PValues <- MakeBootstrap(BootstrapData = as.matrix(ComparisonExpression), 
                    BootstrapNumber = ECGObject@BootstrapNumber,
                    BootstrapGroupSize = as.numeric(GroupFunction[[1]][1]),
                    BootstrapSeed = 
                    ECGObject@SeedNumber*as.numeric(GroupFunction[[1]][1]))

    ResultPValues <- GroupFunction
    ResultPValues <- lapply(ResultPValues, function(RPValues, PValues){
            RPValues$pValue_h <- sum(PValues[,1] > as.numeric(RPValues[6]))/
                                    ECGObject@BootstrapNumber
            RPValues$pValue_n <- sum(PValues[,2] > as.numeric(RPValues[7]))/
                                    ECGObject@BootstrapNumber
            return(RPValues)}, PValues = PValues)

    return(ResultPValues)
}

########
## Function to run Wilcoxon test.
########
WCAnalysis <- function(pValuesGroups){
    GroupData <- pValuesGroups[,c(2,3)]
    if(length(unique(GroupData[,1][GroupData[,1]>0]))==length(GroupData[,1]) &
        length(unique(GroupData[,2][GroupData[,2]>0]))==length(GroupData[,2])
        & length(unique(c(GroupData[,1][GroupData[,1]>0],
        GroupData[,2][GroupData[,2]>0])))==(length(GroupData[,2])*2) &
        length(GroupData[,1]>1)){
        return(wilcox.test(GroupData[,1],GroupData[,2], paired=TRUE)$p.value)
    }else{
        return(1)
    }
}

########
## Function to run the Fisher test.
########
FSHAnalysis <- function(pValuesGroups,Comparison){
    INGroupGenes <- mutate(pValuesGroups,relative = pValuesGroups[,3]/
(pValuesGroups[,2]+pValuesGroups[,3]))
    OUTGroupGenes <- filter(Comparison,!(as.character(Comparison[,1]) %in%
                            as.character(INGroupGenes[,1])))
    OUTGroupGenes <- mutate(OUTGroupGenes,relative = OUTGroupGenes[,3]/
                            (OUTGroupGenes[,2]+OUTGroupGenes[,3]))

    return(fisher.test(matrix(c(sum(INGroupGenes$relative>0.5,na.rm = TRUE),
                                sum(OUTGroupGenes$relative>0.5,na.rm = TRUE),
                                sum(INGroupGenes$relative<0.5,na.rm = TRUE),
                                sum(OUTGroupGenes$relative<0.5,na.rm = TRUE)),
                                2,2))$p.value)
}

############
#Arguments checking and object building
############
ECGMainData <- function(ComparisonID,
                        ExpressionData,
                        MinGene,
                        MaxGene,
                        SeedNumber,
                        BootstrapNumber,
                        PCorrection,
                        DBSpecies,
                        PCorrectionMethod,
                        WilcoxonTest,
                        FisherTest,
                        AnalysisDomain,
                        GeneIdentifier,
                        completeTest){

    #Check the expression data. ExpressionData must be a data frame or a path
    #for a text file tab separated containing at least 3
    #columns. First column mandatory corresponds to gene names,
    #according to GeneIdentifier argument. Second, third and the
    #others correspond to the gene samples expression.
    if(!is.null(ExpressionData)){
        GeneExpressionData <- .checkExpressionData(ExpressionData)
    }else{
        stop("Please inform a valid expression data!")
    }

    #Check the comparison IDs. ComparisonID argument must be a vector in wich 
    #each element corresponds to 2 sample columns from
    #the expression data. The data sample columns in each element from the
    #vector are comma separated.
    
    if(!is.null(ComparisonID)){
        ComparisonID <- .checkComparisonID(ComparisonID,GeneExpressionData)
    }else{
        stop("Please inform a valid comparison ID!")
    }

    #Check the minimum and maximum number of genes per group of a GFAG 
    #(Group of Functionally Associated Genes). Both must be integer
    #positive values and different from zero. Besides, the argument MaxGene 
    #must be allways greater than MinGene.
    if(!is.null(MinGene) & !is.null(MaxGene)){
        GeneNumbers <- .checkGeneNumbers(MinGene,MaxGene)
        InfGeneLimit <- GeneNumbers[1]
        SupGeneLimit <- GeneNumbers[2]
    }
    if(is.null(MinGene)){
        InfGeneLimit <- 3
    }
    if(is.null(MaxGene)){
        SupGeneLimit <- 2000
    }

    #Check the domain analysis, species reference database and gene 
    #identifier. The argument AnalysisDomain must be a character
    #corresponding to a domain (gobp, gocc, gomf, kegg or own). The argument
    #DBSpecies must be a character corresponding to an
    #OrgDb species package (org.Hs.eg.db, org.Dm.eg.db ...) or a character 
    #path for an own gene annotation file containing 3 columns:
    #gene name, term annotation and description of the term annotation. 
    #The GeneIdentifier argument must be a character containing
    #the nomenclature to be used (symbol or entrez).
    if(!is.null(AnalysisDomain) & !is.null(DBSpecies) & 
        !is.null(GeneIdentifier)){
        Analysis<-.checkAnalysisDomain_DBSpecies_GeneIdentifier(
                    AnalysisDomain,DBSpecies,GeneIdentifier,
                    GeneExpressionData)
        DomainGroup <- Analysis[[1]]
        DataSpeciesFunctionsSample <- Analysis[[2]]
        DataSpeciesFunctionsRaw <- Analysis[[3]]
        GeneNomenclature <- Analysis[[4]]
    }
    if(is.null(AnalysisDomain)){
        stop("Please inform a valid domain!")
    }
    if(is.null(DBSpecies)){
        stop("Please inform a valid database species!")
    }
    if(is.null(GeneIdentifier)){
        stop("Please inform a valid gene identifier!")
    }

    if(completeTest){
        #Check the seed for random generation numbers. The argument SeedNumber
        #must be a numeric value allways positive, greater than or
        #equal to zero.
        if(!is.null(SeedNumber)){
            SeedNumber <- .checkSeedNumber(SeedNumber)
        }else{
            SeedNumber <- 10049
        }
        
        #Check the number of bootstraps necessary for defining GFAG p-values.
        #The argument BootstrapNumber must be an integer number
        #greater than zero.
        if(!is.null(BootstrapNumber)){
            BootstrapNumber <- .checkBootstrapNumber(BootstrapNumber)
        }else{
            BootstrapNumber <- 1000
        }
        
        #Check the cutoff to be used for one of the p-value correction 
        #methods. The PCorrection argument must be a numeric value between
        #zero and one.
        if(!is.null(PCorrection)){
            PCorrection <- .checkPCorrection(PCorrection)
        }else{
            PCorrection <- 0.05
        }
        
        #Check the p-value correction method. The PCorrectionMethod argument
        #must be a character corresponding to one of the p.adjust function
        #correction methods (holm, hochberg, hommel, bonferroni, BH, BY, fdr).
        if(!is.null(PCorrectionMethod)){
            PCorrectionMethod <- .checkPCorrectionMethod(PCorrectionMethod)
        }else{
            PCorrectionMethod <- "fdr"
        }
        
        #Check if it will be performed the Wilcoxon Rank Sum Test. The 
        #WilcoxonTest argument should be TRUE for running the test or FALSE
        #if the test won't be performed.
        if(!is.null(WilcoxonTest)){
            WilcoxonTest <- .checkWilcoxonTest(WilcoxonTest)
        }else{
            WilcoxonTest <- FALSE
        }
        
        #Check if it will be performed the Fisher Exact Test. The FisherTest
        #argument should be TRUE for running the test or FALSE
        #if the test won't be performed.
        if(!is.null(FisherTest)){
            FisherTest <- .checkFisherTest(FisherTest)
        }else{
            FisherTest <- FALSE
        }
        
        #Object building, according to the necessary arguments for running 
        #complete analysis ADAM.
        inputObject <- new(Class = "ECGMainData",
                        ComparisonID = ComparisonID,
                        ExpressionData = GeneExpressionData,
                        MinGene = InfGeneLimit,
                        MaxGene = SupGeneLimit,
                        SeedNumber = SeedNumber,
                        BootstrapNumber = BootstrapNumber,
                        PCorrection = PCorrection,
                        DBSpeciesFunctionsSample = DataSpeciesFunctionsSample,
                        DBSpeciesFunctionsRaw = DataSpeciesFunctionsRaw,
                        PCorrectionMethod = PCorrectionMethod,
                        WilcoxonTest = WilcoxonTest,
                        FisherTest = FisherTest,
                        AnalysisDomain = DomainGroup,
                        GeneIdentifier = GeneNomenclature)
    }else{
        #Object building, according to the necessary arguments for running 
        #parcial analysis with ADAM.
        inputObject <- new(Class = "ECGMainData",
                        ComparisonID = ComparisonID,
                        ExpressionData = GeneExpressionData,
                        MinGene = InfGeneLimit,
                        MaxGene = SupGeneLimit,
                        DBSpeciesFunctionsSample = DataSpeciesFunctionsSample,
                        DBSpeciesFunctionsRaw = DataSpeciesFunctionsRaw,
                        AnalysisDomain = DomainGroup,
                        GeneIdentifier = GeneNomenclature)
    }
    return(inputObject)
}

#' @title Group of Functionally Associated Genes (GFAG) complete analysis
#' @rdname GFAGAnalysis
#' @import Rcpp GO.db KEGGREST
#' @importFrom SummarizedExperiment assay
#' @import pbapply methods knitr
#' @importFrom utils installed.packages
#' @importFrom Rcpp evalCpp
#' @importFrom stringr str_split
#' @importFrom stats wilcox.test fisher.test
#' @importFrom DT datatable
#' @importFrom dplyr arrange mutate filter
#' @useDynLib ADAM
#' @importFrom stats p.adjust
#' @importFrom utils read.table write.table
#' @usage GFAGAnalysis(ComparisonID, ExpressionData, MinGene, MaxGene, 
#' SeedNumber, BootstrapNumber, PCorrection, DBSpecies, PCorrectionMethod, 
#' WilcoxonTest, FisherTest, AnalysisDomain, GeneIdentifier)
#' @description Analysis of functionally associated gene groups, based on
#' gene diversity and activity, for different species according to existing 
#' annotation packages or user's personal annotations. GFAGAnalysis function 
#' allows to run a complete analysis, using all available arguments. 
#' @param ComparisonID Sample comparisons identification. It must be a vector
#' in which each element corresponds to 2 sample columns from the expression 
#' data. The data's sample columns in each element from the vector are comma 
#' separated. The first one refers to the control sample, while the second
#' refers to the experiment. This argument must be informed by the user. There 
#' is no default value for it.
#' @param ExpressionData Gene expression data (microarray or RNA-seq, for
#' example). It must be a SummarizedExperiment object, a data frame or a path
#' for a text file tab separated containing at least 3 columns. First column 
#' mandatory corresponds to the gene identification, according to 
#' GeneIdentifier argument. Second, third and the other columns correspond to
#' the gene expression values realated to the genes in the first column and
#' each of these columns correspond to a different sample (control versus 
#' experiment). This argument must be informed by the user. There 
#' is no default value for it.
#' @param MinGene Minimum number of genes per GFAG. It must be a positive
#' integer value different from zero and lower than MaxGene argument.
#' Default is 3.
#' @param MaxGene Maximum number of genes per GFAG. It must be a positive
#' integer value different from zero and greater than MinGene argument.
#' Default is 2000.
#' @param SeedNumber Seed for bootstrap. A numeric positive value used as seed
#' for random number generating by bootstrap function. Default is 1049.
#' @param BootstrapNumber Number of bootstraps. A numeric value greater than 
#' zero, used by bootstrap function generates p-values for each GFAG. Default
#' is 1000. 
#' @param PCorrection Cutoff for p-value correction. A numeric value between 
#' 0 and 1. Default is 0.05.
#' @param DBSpecies A string corresponding to the name of an OrgDb species 
#' gene annotation package: org.Ag.eg.db (Anopheles gambiae), org.At.tair.db 
#' (Arabdopsis thaliana), org.Bt.eg.db (Bos taurus), org.Ce.eg.db
#' (Caenorhabditis elegans), org.Cf.eg.db (Canis familiaris), org.Dm.eg.db 
#' (Drosophila melanogaster), org.Dr.eg.db (Danio rerio), org.EcK12.eg.db 
#' (Escherichia coli K12), org.EcSakai.eg.db (Escherichia coli Sakai),
#' org.Gg.eg.db (Gallus gallus), org.Hs.eg.db (Homo sapiens), org.Mm.eg.db
#' (Mus musculus), org.Mmu.eg.db (Macaca mulatta), org.Pf.plasmo.db
#' (Plasmodium falciparum), org.Pt.eg.db (Pan troglodytes), org.Rn.eg.db
#' (Rattus norvegicus), org.Sc.sgd.db (Saccharomyces cerevisiae),
#' org.Ss.eg.db (Sus scrofa) and org.Xl.eg.db (Xenopus laevis).
#' If there is no package, it's possible for the user to create a personal 
#' gene annotation file, tab separated, containing 3 columns: gene, term 
#' annotation code and description of the term annotation. So, istead of a 
#' string with an OrgDb name, inform a data frame or a path for the file.
#' This argument must be informed by the user. There is no default value for
#' it.
#' @param PCorrectionMethod Method p-value correction: holm, hochberg, hommel,
#' bonferroni, bh, by or fdr. Default is "fdr".
#' @param WilcoxonTest A logical value indicating whether or not to perform
#' Wilcoxon test (TRUE for performing and FALSE for not performing).
#' Default is FALSE.
#' @param FisherTest  A logical value indicating whether or not to perform
#' Fisher's exact test (TRUE for performing and FALSE for not performing).
#' Default is FALSE.
#' @param AnalysisDomain Analysis domain to be considered for building GFAGs, 
#' according: gobp (Gene Ontology - Biological Processes), gocc (Gene Ontology
#'  - Celular Components), gomf (Gene Ontology - Molecular Functions), kegg
#' (KEGG Pathways) or own (if there is no annotation package - the annotations 
#' were defined in a file by the user). This argument must be informed by the 
#' user. There is no default value for it.
#' @param GeneIdentifier Gene nomenclature to be used: entrez or tairID
#' for Arabdopsis thaliana, entrez or orfID for Saccharomyces cerevisiae, 
#' symbol or orfID for Plasmodium falciparum and symbol or entrez for the 
#' other species. If there is no annotation package, just put the 
#' gene nomenclature present in the user's personal annotations. This argument 
#' must be informed by the user. There is no default value for it.
#' @details The genes present in the expression data are grouped by their 
#' respective functions according to the domains described by 
#' AnalysisDomain argument. The relationship between genes and functions are 
#' made based on the species annotation package. If there is no annotation 
#' package, a three column file (gene, function and function description) must
#' be provided. For each GFAG, gene diversity and activity in each sample are 
#' calculated. As the package always compare two samples (control versus 
#' experiment), relative gene diversity and activity for each GFAG are
#' calculated. Using bootstrap method, for each GFAG, according to relative 
#' gene diversity and activity, two p-values are calculated. The p-values are 
#' then corrected, according to the correction method defined by 
#' PCorrectionMethod argument, generating a q-value. The significative GFAGs 
#' will be those whoose q-value stay under the cutoff set by PCorrection 
#' argument. Optionally, it's possible to run Wilcoxon test and/or Fisher's
#' exact test. These tests also provide a corrected p-value, 
#' and siginificative groups can be seen through them.
#' @author André Luiz Molan (andre.molan@unesp.br)
#' @references CASTRO, M. A., RYBARCZYK-FILHO, J. L., et al. Viacomplex:
#' software for landscape analysis of gene expression networks in genomic
#' context. Bioinformatics, Oxford Univ
#' Press, v. 25, n. 11, p. 1468–1469, 2009.
#' @return Return a list with two elements. The first one refers to a 
#' data frame with the GFAGs and their respective genes. The second one is a 
#' a list where each position is a data frame presenting the result of the 
#' analysis, according to ComparisonID argument.
#' @examples
#' ##
#' #Complete Analysis with Aedes aetypti through GFAGAnalysis function
#' ##
#' data(ExpressionAedes)
#' data(KeggPathwaysAedes)
#' ResultAnalysis <- GFAGAnalysis(ComparisonID = c("control1,experiment1", 
#' "control2,experiment2"), ExpressionData = ExpressionAedes, MinGene = 3L,
#' MaxGene = 20L, SeedNumber = 1049, BootstrapNumber = 1000L,
#' PCorrection = 0.05, DBSpecies = KeggPathwaysAedes,
#' PCorrectionMethod = "fdr", WilcoxonTest = TRUE, FisherTest = TRUE,
#' AnalysisDomain = "own", GeneIdentifier = "gene")
#' \dontrun{
#' head(ResultAnalysis[[1]]) #Relation between genes and functions
#' head(ResultAnalysis[[2]][1]) #Result comparison 1
#' head(ResultAnalysis[[2]][2]) #Result comparison 2
#' }
#' @export

GFAGAnalysis <- function(ComparisonID = NULL, ExpressionData = NULL,
                        MinGene = 3, MaxGene = 2000, SeedNumber = 1049,
                        BootstrapNumber = 1000, PCorrection = 0.05,
                        DBSpecies = NULL, PCorrectionMethod = "fdr",
                        WilcoxonTest = FALSE, FisherTest = FALSE,
                        AnalysisDomain = NULL, GeneIdentifier = NULL){
    message("Creating object ...")
    
    ECGObject <- ECGMainData(ComparisonID = ComparisonID, ExpressionData =
                                ExpressionData, MinGene = MinGene,
                                MaxGene = MaxGene,
                                SeedNumber = SeedNumber, BootstrapNumber = 
                                BootstrapNumber, PCorrection = PCorrection,
                                DBSpecies = DBSpecies, PCorrectionMethod = 
                                PCorrectionMethod, WilcoxonTest=WilcoxonTest,
                                FisherTest = FisherTest, AnalysisDomain =
                                AnalysisDomain,
                                GeneIdentifier = GeneIdentifier,
                                completeTest = TRUE)
    #GFAG Analysis
    ResultAnalysis <- lapply(ECGObject@ComparisonID, FUN = makeAnalysis,
                            ECGObject = ECGObject, completeTest = TRUE)
    
    #File relating genes and functions
    GenesFile <- as.data.frame(do.call(rbind,
                lapply(ECGObject@DBSpeciesFunctionsSample,
                    function(DBFunctionsSample){
                    as.data.frame(as.vector(unlist(DBFunctionsSample)))})))
    GenesFile$GroupID  <- row.names(GenesFile)
    GroupID <- data.frame(do.call('rbind',
                        strsplit(as.character(GenesFile$GroupID),
                        '<==>',fixed=TRUE)))
    rownames(GenesFile) <- NULL
    colnames(GenesFile) <- c("gene","GroupID")
    GenesFile$GroupID <- trimws(GroupID$X1)
    GenesFile <- unique(GenesFile)

    return(list(GenesFile,ResultAnalysis))
}

#' @title Group of Functionally Associated Genes (GFAG) partial analysis
#' @rdname ADAnalysis
#' @usage ADAnalysis(ComparisonID, ExpressionData, MinGene, MaxGene, 
#' DBSpecies, AnalysisDomain, GeneIdentifier)
#' @description Analysis of functionally associated gene groups, based on
#' gene diversity and activity, for different species according to existing 
#' annotation packages or user's personal annotations. ADAnalysis function 
#' allows to run a partial analysis, where is calculated just gene diversity
#' and activity of each GFAG with no signicance by bootrstrap, Wilcoxon or 
#' Fisher.
#' @param ComparisonID Sample comparisons identification. It must be a vector
#' in which each element corresponds to 2 sample columns from the expression 
#' data. The data's sample columns in each element from the vector are comma 
#' separated. The first one refers to the control sample, while the second
#' refers to the experiment. This argument must be informed by the user. There 
#' is no default value for it.
#' @param ExpressionData Gene expression data (microarray or RNA-seq, for
#' example). It must be a SummarizedExperiment object, a data frame or a path
#' for a text file tab separated containing at least 3 columns. First column 
#' mandatory corresponds to the gene identification, according to 
#' GeneIdentifier argument. Second, third and the other columns correspond to
#' the gene expression values realated to the genes in the first column and
#' each of these columns correspond to a different sample (control versus 
#' experiment). This argument must be informed by the user. There 
#' is no default value for it.
#' @param MinGene Minimum number of genes per GFAG. It must be a positive
#' integer value different from zero and lower than MaxGene argument.
#' Default is 3.
#' @param MaxGene Maximum number of genes per GFAG. It must be a positive
#' integer value different from zero and greater than MinGene argument.
#' Default is 2000.
#' @param DBSpecies A string corresponding to the name of an OrgDb species 
#' gene annotation package: org.Ag.eg.db (Anopheles gambiae), org.At.tair.db 
#' (Arabdopsis thaliana), org.Bt.eg.db (Bos taurus), org.Ce.eg.db
#' (Caenorhabditis elegans), org.Cf.eg.db (Canis familiaris), org.Dm.eg.db 
#' (Drosophila melanogaster), org.Dr.eg.db (Danio rerio), org.EcK12.eg.db 
#' (Escherichia coli K12), org.EcSakai.eg.db (Escherichia coli Sakai),
#' org.Gg.eg.db (Gallus gallus), org.Hs.eg.db (Homo sapiens), org.Mm.eg.db
#' (Mus musculus), org.Mmu.eg.db (Macaca mulatta), org.Pf.plasmo.db
#' (Plasmodium falciparum), org.Pt.eg.db (Pan troglodytes), org.Rn.eg.db
#' (Rattus norvegicus), org.Sc.sgd.db (Saccharomyces cerevisiae),
#' org.Ss.eg.db (Sus scrofa) and org.Xl.eg.db (Xenopus laevis).
#' If there is no package, it's possible for the user to create a personal 
#' gene annotation file, tab separated, containing 3 columns: gene, term 
#' annotation code and description of the term annotation. So, istead of a 
#' string with an OrgDb name, inform a data frame or a path for the file.
#' This argument must be informed by the user. There is no default value for
#' it.
#' @param AnalysisDomain Analysis domain to be considered for building GFAGs, 
#' according: gobp (Gene Ontology - Biological Processes), gocc (Gene Ontology
#'  - Celular Components), gomf (Gene Ontology - Molecular Functions), kegg
#' (KEGG Pathways) or own (if there is no annotation package - the annotations 
#' were defined in a file by the user). This argument must be informed by the 
#' user. There is no default value for it.
#' @param GeneIdentifier Gene nomenclature to be used: entrez or tairID
#' for Arabdopsis thaliana, entrez or orfID for Saccharomyces cerevisiae, 
#' symbol or orfID for Plasmodium falciparum and symbol or entrez for the 
#' other species. If there is no annotation package, just put the 
#' gene nomenclature present in the user's personal annotations. This argument 
#' must be informed by the user. There is no default value for it.
#' @details The genes present in the expression data are grouped by their 
#' respective functions according to the domains described by 
#' AnalysisDomain argument. The relationship between genes and functions are 
#' made based on the species annotation package. If there is no annotation 
#' package, a three column file (gene, function and function description) must
#' be provided. For each GFAG, gene diversity and activity in each sample are 
#' calculated. As the package always compare two samples (control versus 
#' experiment), relative gene diversity and activity for each GFAG are
#' calculated.
#' @author André Luiz Molan (andre.molan@unesp.br)
#' @references CASTRO, M. A., RYBARCZYK-FILHO, J. L., et al. Viacomplex:
#' software for landscape analysis of gene expression networks in genomic
#' context. Bioinformatics, Oxford Univ
#' Press, v. 25, n. 11, p. 1468–1469, 2009.
#' @return Return a list with two elements. The first one refers to a 
#' data frame with the GFAGs and their respective genes. The second one is a 
#' a list where each position is a data frame presenting the result of the 
#' analysis, according to ComparisonID argument.
#' @examples
#' ##
#' #Partial Analysis with Aedes aegypti through ADAnalysis function
#' ##
#' data(ExpressionHs)
#' ResultAnalysis <- ADAnalysis(ComparisonID = c("control1,experiment1"),
#' ExpressionData = ExpressionAedes, MinGene = 3L, MaxGene = 20L, 
#' DBSpecies = KeggPathwaysAedes, AnalysisDomain = "own",
#' GeneIdentifier = "geneStableID")
#' \dontrun{
#' head(ResultAnalysis[[1]]) #Relation between genes and functions
#' head(ResultAnalysis[[2]][1]) #Result comparison 1
#' }
#' @export
ADAnalysis <- function(ComparisonID = NULL, ExpressionData = NULL, MinGene=3,
                    MaxGene=2000, DBSpecies = NULL, AnalysisDomain = NULL,
                    GeneIdentifier = NULL){
    message("Creating object ...")
    ECGObject <- ECGMainData(ComparisonID = ComparisonID, 
                            ExpressionData = ExpressionData, 
                            MinGene = MinGene,
                            MaxGene = MaxGene, DBSpecies = DBSpecies, 
                            AnalysisDomain = AnalysisDomain,
                            GeneIdentifier = GeneIdentifier,
                            completeTest = FALSE)
    
    #GFAG Analysis
    ResultAnalysis <- lapply(ECGObject@ComparisonID, FUN = makeAnalysis,
                            ECGObject = ECGObject, completeTest = FALSE)
    
    #File relating genes and functions
    GenesFile <- as.data.frame(do.call(rbind,
                    lapply(ECGObject@DBSpeciesFunctionsSample,
                    function(DBFunctionsSample){
                    as.data.frame(as.vector(unlist(DBFunctionsSample)))})))
    GenesFile$GroupID  <- row.names(GenesFile)
    GroupID <- data.frame(do.call('rbind', 
                                strsplit(as.character(GenesFile$GroupID),
                                '<==>',fixed=TRUE)))
    rownames(GenesFile) <- NULL
    colnames(GenesFile) <- c("gene","GroupID")
    GenesFile$GroupID <- trimws(GroupID$X1)
    GenesFile <- unique(GenesFile)

    return(list(GenesFile,ResultAnalysis))
}

### Tests
test_that("data size from ADAnalysis is ok", {
    expect_silent(data(ExpressionAedes))
    expect_silent(data(KeggPathwaysAedes))
    expect_equal(length(as.data.frame(suppressMessages(
            ADAnalysis(ComparisonID = c("control1,experiment1"),
            ExpressionData = ExpressionAedes, MinGene = 3,
            DBSpecies = KeggPathwaysAedes, MaxGene = 1000,
            AnalysisDomain = "own",
            GeneIdentifier = "geneStableID"))[[2]])$ID),
            length(unique(KeggPathwaysAedes$pathwayID)))
})


test_that("data size from GFAGAnalysis is ok", {
    expect_silent(data(ExpressionAedes))
    expect_silent(data(KeggPathwaysAedes))
    expect_equal(length(as.data.frame(suppressMessages(
            GFAGAnalysis(ComparisonID  = c("control1,experiment1"),
            ExpressionData = ExpressionAedes, MinGene = 3, MaxGene = 20, 
            SeedNumber = 1049, BootstrapNumber = 1000, PCorrection = 0.05,
            DBSpecies = KeggPathwaysAedes, PCorrectionMethod = "fdr", 
            WilcoxonTest = FALSE, FisherTest = FALSE,
            AnalysisDomain = "own",GeneIdentifier="geneStableID"))[[2]])$ID),
            length(unique(KeggPathwaysAedes$pathwayID)))
})

#' @title Aedes aegypti RNA-seq data expression
#' @description A sample fragment of gene expression from an RNA-seq 
#' experiment of Aedes aegypti mosquito.
#' @format A data frame with 34 rows and 5 variables
#' @name ExpressionAedes
#' @references AKBARI, O. S. et al. The developmental transcriptome of the 
#' mosquito aedes aegypti, an invasive species and major arbovirus vector. G3:
#' Genes— Genomes— Genetics, Genetics Society of America, v. 3, n. 9, 
#' p. 1493–1509, 2013.
#' @examples 
#' data(ExpressionAedes)
NULL

#' @title Relation between Aedes aegypti genes and KEGG pathways as 
#' ADAM input
#' @description A relation between the genes in the ExpressionAedes data and
#' their respective KEGG pathways (GFAGs).
#' @format A data frame with 200 rows and 2 variables
#' @name KeggPathwaysAedes
#' @references Molan, A. L. 2018. “Construction of a Tool for Multispecies
#' Genic Functional Enrichment Analysis Among Comparative Samples.”
#' Master’s thesis Institute of Biosciences of Botucatu – Univ. Estadual
#' Paulista. http://hdl.handle.net/11449/157105.
#' @examples
#' data(KeggPathwaysAedes)
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

MakeBootstrap <- function(BootstrapData, BootstrapNumber, BootstrapGroupSize, BootstrapSeed) {
    .Call('_ADAM_MakeBootstrap', PACKAGE = 'ADAM', BootstrapData, BootstrapNumber, BootstrapGroupSize, BootstrapSeed)
}

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
suppressMessages(library(ADAM))

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
data("ExpressionAedes")
head(ExpressionAedes)

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Comparison <- c("control1,experiment1","control2,experiment2")

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Minimum <- 3
Maximum <- 20

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
SeedBootstrap <- 1049
StepsBootstrap <- 1000

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
CutoffValue <- 0.05
MethodCorrection <- "fdr"

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
data("KeggPathwaysAedes")
head(KeggPathwaysAedes)

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Domain <- "own"
Nomenclature <- "geneStableID"

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Wilcoxon <- TRUE
Fisher <- TRUE

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
ResultAnalysis <- suppressMessages(GFAGAnalysis(ComparisonID = Comparison, 
                            ExpressionData = ExpressionAedes,
                            MinGene = Minimum,
                            MaxGene = Maximum,
                            SeedNumber = SeedBootstrap, 
                            BootstrapNumber = StepsBootstrap,
                            PCorrection = CutoffValue,
                            DBSpecies = KeggPathwaysAedes, 
                            PCorrectionMethod = MethodCorrection,
                            WilcoxonTest = Wilcoxon,
                            FisherTest = Fisher,
                            AnalysisDomain = Domain, 
                            GeneIdentifier = Nomenclature))

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
head(ResultAnalysis[[1]])

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
DT::datatable(as.data.frame(ResultAnalysis[[2]][1]), width = 800,
            options = list(scrollX = TRUE))
DT::datatable(as.data.frame(ResultAnalysis[[2]][2]), width = 800, 
            options = list(scrollX = TRUE))

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
suppressMessages(library(ADAM))
data("ExpressionAedes")
data("KeggPathwaysAedes")

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Comparison <- c("control1,experiment1")
Minimum <- 3
Maximum <- 100

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
SpeciesID <- "KeggPathwaysAedes"

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
Domain <- "own"
Nomenclature <- "geneStableID"

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
ResultAnalysis <- suppressMessages(ADAnalysis(ComparisonID = Comparison, 
                            ExpressionData = ExpressionAedes,
                            MinGene = Minimum,
                            MaxGene = Maximum,
                            DBSpecies = KeggPathwaysAedes, 
                            AnalysisDomain = Domain, 
                            GeneIdentifier = Nomenclature))

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
head(ResultAnalysis[[1]])

## ----eval=TRUE, out.height.px=6, out.width.px=6-------------------------------
DT::datatable(as.data.frame(ResultAnalysis[[2]][1]), width = 800, 
            options = list(scrollX = TRUE))

## ----eval=TRUE, out.height.px=6, out.width.px=6, label='Session information', , echo=FALSE----
sessionInfo()

##########
#Argument check functions
##########

if(getRversion() >= "3.5"){
    utils::globalVariables(c("GOID","Ontology","Term","mappedkeys",
                            "org.At.tairENTREZID","org.Sc.sgdENTREZID",
                            "org.Pf.plasmoALIAS2ORF"))
}

.checkExpressionData <- function(ExpressionData){
    
    DataFile <- ExpressionData
    
    if(is(DataFile,"SummarizedExperiment") | 
        is(DataFile,"RangedSummarizedExperiment")){
        DataFile <- as.data.frame(assay(DataFile))
        colnames(DataFile)[1] <- "gene"
        GeneColumn <- as.data.frame(as.character(DataFile$gene))
        DataFile$gene <- NULL
        DataFile <- as.matrix(DataFile)
        DataFile <- apply(DataFile,2,as.numeric)
        DataFile <- as.data.frame(DataFile)
        DataFile <- cbind.data.frame(GeneColumn,DataFile)
    }else if(!is.data.frame(DataFile)){
        if(!file.exists(DataFile)){
            stop("Please, check the format of the expression data. 
                ExpressionData argument must be a valid file or a 
                datra frame.")
        }else{
            DataFile <- read.table(ExpressionData, header = TRUE, sep = "\t",
                                    quote = "", stringsAsFactors = FALSE)
        }
    }
    colnames(DataFile)[1] <- "gene"
    
    if(ncol(DataFile)<3){
        stop("Check the data format to Conditions! The Expression data must 
            have at list 3 columns.")
    }else{
        ExpressionColumns <- DataFile    
        ExpressionColumns$gene <- NULL
        ExpressionColumns <- as.list(ExpressionColumns)
        ExpressionColumns <- sapply(ExpressionColumns,
                                function(ExpColumn) is.numeric(ExpColumn))
        if(sum(ExpressionColumns)!=(ncol(DataFile)-1)){
            stop("Check the format data! Expression values must be numeric.")
        }else if(length(unique(as.character(DataFile$gene))) != 
                length(as.character(DataFile$gene))){
            stop("Gene identifiers must be unique. Check for duplicated
            gene IDs.")
        }
    }
    return(DataFile)
}

.checkComparisonID <- function(ComparisonID,ExpressionData){
    ExpressionData <- .checkExpressionData(ExpressionData)
    ListComparisons <- as.list(ComparisonID)
    ListComparisons <- lapply(ListComparisons,function(CompID) length(unlist(
                                strsplit(CompID,","))))
    ListComparisons <- sum(ifelse(ListComparisons!=2,1,0))
    ExpDataColumns <- colnames(ExpressionData)[-1]
    if(!is.vector(ComparisonID)){
        stop("Check the format data. ComparisonID argument must be a vector!")
    }else if(ListComparisons>0){
        stop("Comparison IDs must have 2 elements!")
    }else if(sum(ifelse(unique(unlist(strsplit(ComparisonID,","))) %in% 
            ExpDataColumns,0,1)) > 0){
        stop("Invalid Comparison IDs! The IDs must correspond to the
                Expression Data column names.")
    }
    return(ComparisonID)
}


.checkGeneNumbers <- function(MinGene,MaxGene){
    if(!is.integer(MinGene) | !is.integer(MaxGene)){
        if(!is.numeric(MinGene) | !is.numeric(MaxGene)){
            stop("Please, check the format of Conditions! MinGene and MaxGene 
            arguments must be integer.")
        }else{
            MinGene <- as.integer(MinGene)
            MaxGene <- as.integer(MaxGene)
        }
    }
    if(MinGene<=0 | MaxGene<=0){
        stop("Check the Data format to Conditions! The number of genes must be
            positive and different from zero.")
    }else if(MinGene>MaxGene){
        stop("Check the Data format to Conditions! The maximum number of genes
            must be greater than the minimum number.")
    }
    return(c(MinGene,MaxGene))
}

.checkSeedNumber <- function(SeedNumber){
    if(!is.numeric(SeedNumber)){
        stop("Please, check the format of Conditions! SeedNumber argument must
            be numeric.")
    }else if(SeedNumber<0){
            stop("Check the Data format Conditions!The seed must be a positive
                value.")
    }
    return(SeedNumber)
}

.checkBootstrapNumber <- function(BootstrapNumber){
    if(!is.integer(BootstrapNumber)){
        if(!is.numeric(BootstrapNumber)){
            stop("Please, check the format of Conditions! BootstrapNumber
                argument must be integer.")
        }else{
            BootstrapNumber<-as.integer(BootstrapNumber)
        }
    }
    if(BootstrapNumber<=0){
        stop("Check the Data format Conditions! The number of bootstrap steps
            genes must be positive and different from zero.")
    }
    return(BootstrapNumber)
}

.checkPCorrection <- function(PCorrection){
    if(!is.numeric(PCorrection)){
        stop("Please, check the format of Conditions! PCorrection argument
            must be numeric.")
    }else if(PCorrection<0 || PCorrection>1){
        stop("Check the Data format Conditions! Correction value must be 
            between zero and one.")
    }
    return(PCorrection)
}

.checkPCorrectionMethod <- function(PCorrectionMethod){
    options <- c("holm", "hochberg", "hommel", "bonferroni", "bh", "by","fdr")
    OptionsPAdjust <- c("holm", "hochberg", "hommel", "bonferroni", "BH",
                        "BY","fdr")
    if  (!(tolower(PCorrectionMethod) %in% options)){
        stop(paste("Correction method",PCorrectionMethod,"not found."))
    }
    return(OptionsPAdjust[grep(tolower(PCorrectionMethod),options)])
}

.checkWilcoxonTest <- function(WilcoxonTest){
    if(!is.logical(WilcoxonTest)){
        stop("Please, check the format of Conditions! WilcoxonTest argument 
            must be logical.")
    }
    return(WilcoxonTest)
}

.checkFisherTest <- function(FisherTest){
    if(!is.logical(FisherTest)){
        stop("Please, check the format of Conditions! FisherTest argument must
            be logical.")
    }
    return(FisherTest)
}

###
# Creates a data frame relating Species and gene annotation package. This 
# data frame is used by .checkAnalysisDomain_DBSpecies_GeneIdentifier funcion.
###
.CreateDataSpecies <-function(){
    SpeciesDescription <- c("Anopheles gambiae", "Arabdopsis thaliana",
                        "Bos taurus", "Caenorhabditis elegans",
                        "Canis familiaris", "Drosophila melanogaster",
                        "Danio rerio", "Escherichia coli K12",
                        "Escherichia coli Sakai", "Gallus gallus",
                        "Homo sapiens", "Mus musculus", "Macaca mulatta",
                        "Plasmodium falciparum", "Pan troglodytes",
                        "Rattus norvegicus", "Saccharomyces cerevisiae",
                        "Sus scrofa","Xenopus laevis")
    SpeciesID <- c("aga","ath","bta","cel","cfa","dme","dre","eco","ecs",
                "gga","hsa","mmu","mcc","pfa","ptr","rno","sce","ssc","xla")
    DBSpeciesList <- c("org.Ag.eg.db", "org.At.tair.db", "org.Bt.eg.db",
                    "org.Ce.eg.db", "org.Cf.eg.db","org.Dm.eg.db",
                    "org.Dr.eg.db", "org.EcK12.eg.db", "org.EcSakai.eg.db",
                    "org.Gg.eg.db", "org.Hs.eg.db", "org.Mm.eg.db",
                    "org.Mmu.eg.db", "org.Pf.plasmo.db", "org.Pt.eg.db",
                    "org.Rn.eg.db", "org.Sc.sgd.db", "org.Ss.eg.db",
                    "org.Xl.eg.db")

    DataFrameSpecies <- data.frame(DBSpeciesList, SpeciesDescription,
                                    SpeciesID)
    return(DataFrameSpecies)
}

.checkAnalysisDomain_DBSpecies_GeneIdentifier <- function(AnalysisDomain,
                                                        DBSpecies,
                                                        GeneIdentifier,
                                                        GeneExpressionData){
    #Creates a data frame relating Species and gene annotation packages
    DataFrameSpecies <- .CreateDataSpecies()
    
    #Available domains
    options <- c("gobp","gocc","gomf","kegg","own")
    
    #Available nomenclatures
    GeneNomenclatures <- c("symbol","entrez","tair","orf")
    
    SpeciesID <- DBSpecies
    if(!(tolower(AnalysisDomain) %in% options)){
        stop(paste("Analysis domain",AnalysisDomain,"does not exist!"))
    }else if(AnalysisDomain!=options[5]){
        if(!is.data.frame(SpeciesID)){
            if(!file.exists(SpeciesID)){
                if(!SpeciesID %in% DataFrameSpecies$DBSpeciesList){
                    if(tolower(SpeciesID)=="eck12"){
                        SpeciesID <- "EcK12"
                    }else if(tolower(SpeciesID)=="ecsakai"){
                        SpeciesID <- "EcSakai"
                    }else if(tolower(SpeciesID)=="mmu"){
                        SpeciesID <- "Mmu"
                    }else{
                        SpeciesID <- unlist(strsplit(SpeciesID,""))
                        SpeciesID <- paste0(toupper(SpeciesID[1]),
                            paste0(tolower(SpeciesID[2:length(SpeciesID)]),
                            collapse = ""))
                    }
                    SpeciesID <- ifelse(length(DataFrameSpecies$DBSpeciesList[
                        grep(SpeciesID,DataFrameSpecies$DBSpeciesList)])==0,
                        DBSpecies,
                        as.character(DataFrameSpecies$DBSpeciesList[
                        grep(SpeciesID,DataFrameSpecies$DBSpeciesList)]))
                    if(!SpeciesID %in% DataFrameSpecies$DBSpeciesList){
                        stop("Gene annotation package does not supported! 
                            Please check it.")
                    }
                }
            }else{
                stop("Please inform a valid annotation package name!")
            }
        }else{
            stop("Please inform a valid annotation package name!")
        }
    }

    ExpressionData <- GeneExpressionData
    if(!(is.data.frame(GeneExpressionData))){
        ExpressionData <- read.table(GeneExpressionData, sep = "\t",
                                    header = TRUE, quote = "",
                                    stringsAsFactors = FALSE)
    }
    colnames(ExpressionData)[1] <- "gene"
    
    if(tolower(AnalysisDomain)==options[5]){
        if(!is.data.frame(SpeciesID)){
            if(!file.exists(SpeciesID)){
                stop("Own functions file does not exist!")
            }else{
                DBFile <- read.table(SpeciesID, header = TRUE,
                                    sep = "\t", quote = "",
                                    stringsAsFactors = FALSE)
                colnames(DBFile) <- c("gene","ID","Description")
                DBFile <- .checkDB(DBFile = DBFile,
                        ExpressionData = ExpressionData)
            }
        }else{
            DBFile <- SpeciesID
            colnames(DBFile) <- c("gene","ID","Description")
            DBFile <- .checkDB(DBFile=DBFile,
                                ExpressionData = ExpressionData)
        }
    }else if(.checkPackage(NamePackage = SpeciesID)){
            if(!GeneIdentifier %in% GeneNomenclatures){
                stop("Gene identifier invalid!")
            }else if(GeneIdentifier==GeneNomenclatures[3] & 
                    SpeciesID!="org.At.tair.db"){
                stop("Incorrect GeneIdentifier!")
            }else if(GeneIdentifier==GeneNomenclatures[4] & 
                    SpeciesID!="org.Sc.sgd.db" & 
                    SpeciesID!="org.Pf.plasmo.db"){
                    stop("Incorrect GeneIdentifier!")
            }else{
                if(SpeciesID=="org.At.tair.db"){
                    SymbolEGClass <- org.At.tairENTREZID #EntrezID and tairID
                    SymbolEGList <- unlist(as.list(SymbolEGClass[mappedkeys(
                                    SymbolEGClass)]))
                    SymbolEGList <-data.frame(tair=names(SymbolEGList),
                                            entrez=as.vector(SymbolEGList))
            }else if(SpeciesID=="org.Sc.sgd.db"){
                    SymbolEGClass <- org.Sc.sgdENTREZID #EntrezID/ORF numbers
                    SymbolEGList <- unlist(as.list(SymbolEGClass[mappedkeys(
                                    SymbolEGClass)]))
                    SymbolEGList <-data.frame(orf=names(SymbolEGList),
                                    entrez=as.vector(SymbolEGList))
            }else if(SpeciesID=="org.Pf.plasmo.db"){
                    SymbolEGClass<-org.Pf.plasmoALIAS2ORF #GeneSymbol / ORF ID
                    SymbolEGList <- unlist(as.list(SymbolEGClass[mappedkeys(
                                    SymbolEGClass)]))
                    SymbolEGList <-data.frame(symbol=names(SymbolEGList),
                                    orf=as.vector(SymbolEGList))
            }else{
                SymbolEGClass <- get(paste0("org.",unlist(strsplit(SpeciesID,
                                "[.]"))[2],".egSYMBOL2EG")) #GeneSymbol/Entrez
                SymbolEGList <- unlist(as.list(SymbolEGClass[mappedkeys(
                                SymbolEGClass)]))
                SymbolEGList <-data.frame(symbol=names(SymbolEGList),
                                entrez=as.vector(SymbolEGList))
                
                if(SpeciesID=="org.Ag.eg.db"){
                    GenesAg<-str_split(as.character(SymbolEGList[,1]),"AgaP_")
                    GenesAg <- lapply(GenesAg, function(GeneNameAg){
                        if(length(GeneNameAg) > 1){
                            return(GeneNameAg[2])
                        }else{
                            return(GeneNameAg[1])
                        }
                    })
                    SymbolEGList$symbol <- unlist(GenesAg)
                }
            }

            if(sum(ifelse(ExpressionData$gene %in% 
                            SymbolEGList[,c(grep(tolower(GeneIdentifier),
                            colnames(SymbolEGList)))] |
                            tolower(ExpressionData$gene) %in% 
                            SymbolEGList[,c(grep(tolower(GeneIdentifier),
                            colnames(SymbolEGList)))] |
                            toupper(ExpressionData$gene) %in% 
                            SymbolEGList[,c(grep(tolower(GeneIdentifier),
                            colnames(SymbolEGList)))],1,0)) < 1){
                stop("The genes in the Expression Data are not in the species 
                    database. Check the gene identifiers!")
            }else{
                DBFile <- .checkGenesDomain(AnalysisDomain = 
                                                    tolower(AnalysisDomain), 
                DBSpecies = SpeciesID, 
                IDGene = tolower(GeneIdentifier), 
                ListGenes = SymbolEGList,
                keggID=as.character(DataFrameSpecies$SpeciesID[grep(SpeciesID,
                                    DataFrameSpecies$DBSpeciesList)]),
                ExpressionData = ExpressionData)
            }
        }
    }

    return(list(options[grep(tolower(AnalysisDomain),options)],DBFile[[1]],
            DBFile[[2]], tolower(GeneIdentifier)))
}

.checkPackage <- function(NamePackage){
    if(!suppressMessages(requireNamespace(NamePackage))){
        stop(paste("Annotation package",NamePackage,"not availlable.
                    Please install it!"))
    }else{
        suppressMessages(require(NamePackage, character.only = TRUE,
                                warn.conflicts = FALSE))
    }
    return(TRUE)
}

.checkDB <- function(DBFile,ExpressionData){
    if(ncol(DBFile)!=3){
        stop("Own functions file must have 3 columns!")
    }else if(nrow(DBFile)<1){
        stop("Own functions file is empty!")
    }else if(sum(ifelse(ExpressionData$gene %in% DBFile$gene | 
        tolower(ExpressionData$gene) %in% DBFile$gene |
        toupper(ExpressionData$gene) %in% DBFile$gene,1,0)) < 1){
        stop("The genes in the Expression Data are not in the species
            database. Check the gene identifiers!")
        }else if(length(colnames(DBFile))==length(colnames(ExpressionData))){
                if(sum(colnames(DBFile)==colnames(ExpressionData))==3){
                    stop("Check the ExpressionData and DBSpecies arguments.
                        The column names of both files are the same!!")
                }
    }

    DBFunctionsList <- lapply(as.vector(unique(as.character(DBFile$ID))), 
                        FUN = .ListConversionDB, DBFile = DBFile)
    names(DBFunctionsList) <- as.vector(unique(as.character(DBFile$ID)))
    TermOntologies <- DBFile
    TermOntologies$gene <- NULL 
    TermOntologies <- unique(TermOntologies)

    DBFunctionsListSample <- lapply(DBFunctionsList, function(ListElement,
                                    SampleGenes) {SampleGenes[SampleGenes %in% 
                                    ListElement]},
                                    SampleGenes = ExpressionData$gene)
    names(DBFunctionsListSample) <- vapply(names(DBFunctionsList),
                function(ListElement,TermOntologies){
                SubTermOntologies<-subset(TermOntologies,
                            TermOntologies$ID == ListElement)
                return(paste0(as.character(SubTermOntologies$ID),
                    " <==> ",as.character(SubTermOntologies$Description)))},
                TermOntologies=TermOntologies, FUN.VALUE = character(1))

    DBFunctionsListRaw <- lapply(DBFunctionsList,function(ListElement){
                                length(ListElement)})
    names(DBFunctionsListRaw) <- names(DBFunctionsList)

    return(list(DBFunctionsListSample,DBFunctionsListRaw))
}

.ListConversionDB <- function(ListElement,DBFile){
    ListResult <- subset(DBFile, DBFile$ID == ListElement)
    ListResult <- ListResult$gene
    return(ListResult)
}

.checkGenesDomain<-function(AnalysisDomain,DBSpecies,IDGene,ListGenes,keggID,
                            ExpressionData){

    #Each annotation package has its own set of functions that relates genes
    #to ontologies and kegg pathways. This way, we create a list linking the 
    #package to its functions. Besides that, the list also presents the 
    #default and alternative nomenclature supported according to each DB 
    #species. For example, let's consider Arabdopsis thaliana: List element
    #name: org.At.tair.db; Function to relate genes to gene ontologies terms:
    #org.At.tairGO2ALLTAIRS; Default gene nomenclature: "tair" ID;
    #Alternative gene nomenclature: "entrez" ID; Function to relate genes to
    #kegg pathways: org.At.tairARACYC.
    ListDBPackages <- list(org.At.tair.db = c("org.At.tairGO2ALLTAIRS","tair",
                                            "entrez","org.At.tairPATH2TAIR"),
                org.Sc.sgd.db = c("org.Sc.sgdGO2ALLORFS","orf",
                                "entrez","org.Sc.sgdPATH2ORF"),
                org.Pf.plasmo.db = c("org.Pf.plasmoGO2ALLORFS","orf",
                                "symbol","org.Pf.plasmoPATH2ORF"),
                otherDB=c(paste0("org.",unlist(strsplit(DBSpecies,"[.]"))[2],
                        ".egGO2ALLEGS"),"entrez","symbol",
                        paste0("org.",unlist(strsplit(DBSpecies,"[.]"))[2],
                        ".egPATH2EG")))

    if(DBSpecies %in% names(ListDBPackages)){
        OntologyFunction <- c(ListDBPackages[[grep(DBSpecies,names(
                            ListDBPackages))]][1],
                            ListDBPackages[[grep(DBSpecies,names(
                            ListDBPackages))]][2],
                            ListDBPackages[[grep(DBSpecies,names(
                            ListDBPackages))]][3],
                            ListDBPackages[[grep(DBSpecies,names(
                            ListDBPackages))]][4])
    }else{
        OntologyFunction <- c(ListDBPackages[[4]][1],ListDBPackages[[4]][2],
        ListDBPackages[[4]][3],ListDBPackages[[4]][4])
    }

    if(AnalysisDomain == "kegg"){
        keggPathways <- as.data.frame(keggList("pathway", keggID))
        colnames(keggPathways)[1] <- "ID"
        keggPathwaysID <- unlist(lapply(rownames(keggPathways),
                        function(KPathway)
                        unlist(strsplit(KPathway,paste0(":",keggID)))[2]))
        
        keggPathwaysDescription<-unlist(lapply(as.character(keggPathways$ID),
                                function(KPathway) unlist(strsplit(KPathway,
                                paste0("- ",keggID)))[1]))
        
        TermOntologies <- data.frame(ID = keggPathwaysID, 
                                    Description = keggPathwaysDescription)


        DBKeggClass <- get(OntologyFunction[4])
        
        DBFunctionsList <- as.list(DBKeggClass[mappedkeys(DBKeggClass)])

        DBFunctionsList <- lapply(DBFunctionsList, function(DBFunction) 
                                unique(as.vector(DBFunction)))
        
        DBFunctionsList <- mapply(.CheckTerm, ListElement = DBFunctionsList, 
                                ListElementName = names(DBFunctionsList),
                                MoreArgs =  list(VectorTerms = 
                                as.character(TermOntologies$ID)))

        TermOntologies <- TermOntologies[TermOntologies$ID %in% 
                            names(DBFunctionsList),]
        TermOntologies$ID <- paste0(keggID,TermOntologies$ID)

        DBFunctionsList<-DBFunctionsList[lapply(DBFunctionsList, length) > 0]
        names(DBFunctionsList) <- paste0(keggID,names(DBFunctionsList))
    }else{
        infoGO <- GOTERM
        TermOntologies <- data.frame(t(vapply(infoGO, function(GOntology) 
                    c(trimws(GOID(GOntology)), trimws(Ontology(GOntology)), 
                    trimws(Term(GOntology))), FUN.VALUE = character(3))))

        TermOntologies <- subset(TermOntologies, TermOntologies$X2 == 
                            toupper(unlist(strsplit(AnalysisDomain,"go"))[2]))
        colnames(TermOntologies) <- c("ID","Domain","Description")
        TermOntologies$Domain <- NULL
        rownames(TermOntologies) <- NULL

        DBFunctions <- get(OntologyFunction[1])
        DBFunctionsList <- as.list(DBFunctions[mappedkeys(DBFunctions)])
        DBFunctionsList <- lapply(DBFunctionsList, function(DBFunction)
                            unique(as.vector(DBFunction)))
        DBFunctionsList <- mapply(.CheckTerm, ListElement = DBFunctionsList,
                            ListElementName = names(DBFunctionsList),
                            MoreArgs =  list(VectorTerms = 
                            as.character(TermOntologies$ID)))
        DBFunctionsList<-DBFunctionsList[lapply(DBFunctionsList, length) > 0]
    }

    if(IDGene != OntologyFunction[2]){
        DBFunctionsList <- lapply(DBFunctionsList, FUN = .GeneIDConversion, 
                            ListGenes = ListGenes,
                            StandardGeneID = OntologyFunction[2], 
                            AlternativeGeneID = OntologyFunction[3])
    }

    TermOntologies <- subset(TermOntologies, TermOntologies$ID %in%
                        names(DBFunctionsList))

    DBFunctionsListSample <- lapply(DBFunctionsList, 
                            function(ListElement,SampleGenes){
                            SampleGenes[SampleGenes %in% 
                            ListElement]},
                            SampleGenes = ExpressionData$gene)

    CheckSample<-DBFunctionsListSample[lapply(DBFunctionsListSample,
                                                        length) > 0]
    
    if(length(CheckSample) == 0){
        stop(paste("Genes in the Expression Data file are not related to any",
                   AnalysisDomain,"term. Please check the genes in your
                   sample!"))
    }
    
    names(DBFunctionsListSample) <- vapply(names(DBFunctionsList),
                function(ListElement,TermOntologies){
                SubTermOntologies<-subset(TermOntologies, 
                            TermOntologies$ID==ListElement)
                return(paste0(as.character(SubTermOntologies$ID),
                    " <==> ",as.character(SubTermOntologies$Description)))},
                TermOntologies=TermOntologies, FUN.VALUE = character(1))

    
    DBFunctionsListRaw <- lapply(DBFunctionsList,function(ListElement){
                                length(ListElement)})
    names(DBFunctionsListRaw) <- names(DBFunctionsList)

    DBFunctionsListSample<-DBFunctionsListSample[lapply(DBFunctionsListSample,
                                                length) > 0]

    return(list(DBFunctionsListSample,DBFunctionsListRaw))
}

.CheckTerm <- function(ListElement,ListElementName,VectorTerms){
    if(as.character(ListElementName) %in% as.character(VectorTerms)){
        result <- ListElement
    }else{
        result <- NULL
    }
    return(result)
}

.GeneIDConversion <- function(ListElement,ListGenes,StandardGeneID,
                            AlternativeGeneID){
    SubListGenes <- subset(ListGenes, ListGenes[,grep(StandardGeneID,
                    colnames(ListGenes))] %in% ListElement)
    return(as.character(SubListGenes[,grep(AlternativeGeneID,
        colnames(SubListGenes))]))
}

setClass("ECGMainData",
        slots = c(ComparisonID = "vector",
                ExpressionData = "data.frame",
                MinGene = "integer",
                MaxGene = "integer",
                SeedNumber = "numeric",
                BootstrapNumber = "integer",
                PCorrection = "numeric",
                DBSpeciesFunctionsSample = "list",
                DBSpeciesFunctionsRaw = "list",
                PCorrectionMethod = "character",
                WilcoxonTest = "logical",
                FisherTest = "logical",
                AnalysisDomain = "character",
                GeneIdentifier = "character"))

---
title: "ADAM: Activity and Diversity Analysis Module"
author: "André L. Molan, Giordano B. S. Seco, Agnes A. S. Takeda, Jose L. Rybarczyk-Filho"
date: "`r doc_date()`"
package: "`r pkg_ver('ADAM')`"
bibliography: bibliography.bib
fig_caption: yes
output: 
    BiocStyle::html_document:
        css: custom.css
vignette: >
    %\VignetteIndexEntry{"Using ADAM"}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

# Overview
*ADAM* is a GSEA R package created to group a set of genes from 
comparative samples (control *versus* experiment) according to their 
respective functions (Gene Ontology and KEGG pathways as default) and show 
their significance by calculating p-values referring to gene diversity and 
activity (@Castro2009). Each group of genes is called GFAG 
(Group of Functionally Associated Genes). The package has support for many 
species in regards to the genes and their respective functions.

In the package's analysis, all genes present in the expression data are 
grouped by their respective functions according to the domains described by 
AnalysisDomain argument. The relationship between genes and functions are made
based on the species annotation package. If there is no annotation package, 
a three column file (gene, function and function description) must be 
provided. For each GFAG, gene diversity and activity in each sample are 
calculated. As the package always compare two samples (control *versus* 
experiment), relative gene diversity and activity for each GFAG are 
calculated. Using bootstrap method, for each GFAG, according to relative gene
diversity and activity, two p-values are calculated. The p-values are then 
corrected, by using the 
correction method defined by *PCorrectionMethod* argument, generating a q-value 
(@molan2018). The significative GFAGs will be those whose q-value stay under 
the cutoff set by *PCorrection* argument. Optionally, it's possible to run 
Wilcoxon ranck sum test and/or Fisher's exact test (@fontoura2016). These tests
also provide a corrected p-value, and siginificative groups can be seen through 
them.

# GFAGAnalysis
*GFAGAnalysis* function provides a complete analysis, using all available
arguments. As an example, lets consider a gene expression set of 
*Aedes aegypti*:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
suppressMessages(library(ADAM))
```

```{r eval=TRUE, out.height.px=6, out.width.px=6}
data("ExpressionAedes")
head(ExpressionAedes)
```

The first column refers to the gene names, while the others are the expression
obtained by a specific experiment (in this case, RNA-seq). ADAM
always need two samples (control *versus* experiment). This way, we must 
select two sample columns from the expression data for each comparison:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Comparison <- c("control1,experiment1","control2,experiment2")
```

Each GFAG has a number of genes associated to it. This way, the analysis can 
consider all GFAGs or just those with a certain number of genes:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Minimum <- 3
Maximum <- 20
```

The p-values for each GFAG is calculated through the bootstrap method, which
demands a seed for generating random numbers and a number of bootstraps steps
(the number of bootstraps should be a value that ensures the p-value 
precision):

```{r eval=TRUE, out.height.px=6, out.width.px=6}
SeedBootstrap <- 1049
StepsBootstrap <- 1000
```

The p-values will be corrected by a specific method with a certain cutoff 
value:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
CutoffValue <- 0.05
MethodCorrection <- "fdr"
```

In order to group the genes according to their biological functions, it's 
necessary an annotation package or a file relating genes and functions. In this
case, *Aedes aegypti* doesn't have an annotation package. This way, we build our
own file:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
data("KeggPathwaysAedes")
head(KeggPathwaysAedes)
```

It's necessary to inform which function domain and gene nomenclature are being 
used. As *Aedes agypti* doesn't have an annotation package, the domain will be 
"own" and the nomenclature "gene":

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Domain <- "own"
Nomenclature <- "geneStableID"
```

Wilcoxon rank sum test and Fisher's exact test will be run:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Wilcoxon <- TRUE
Fisher <- TRUE
```

As all arguments were defined, then we can run GFAGAnalysis function:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
ResultAnalysis <- suppressMessages(GFAGAnalysis(ComparisonID = Comparison, 
                            ExpressionData = ExpressionAedes,
                            MinGene = Minimum,
                            MaxGene = Maximum,
                            SeedNumber = SeedBootstrap, 
                            BootstrapNumber = StepsBootstrap,
                            PCorrection = CutoffValue,
                            DBSpecies = KeggPathwaysAedes, 
                            PCorrectionMethod = MethodCorrection,
                            WilcoxonTest = Wilcoxon,
                            FisherTest = Fisher,
                            AnalysisDomain = Domain, 
                            GeneIdentifier = Nomenclature))
```

In the example above, we used the function *supressMessages* just to stop 
showing messages during the *GFAGAnalysis* function execution. The output of
*GFAGAnalysis* will be a *list* with two elements. The first corresponds to a
*data frame* showing genes and their respective functions:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
head(ResultAnalysis[[1]])
```

The second element of the output list result corresponds to data frames
according to the argument ComparisonID:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
DT::datatable(as.data.frame(ResultAnalysis[[2]][1]), width = 800,
            options = list(scrollX = TRUE))
DT::datatable(as.data.frame(ResultAnalysis[[2]][2]), width = 800, 
            options = list(scrollX = TRUE))
```

The data frames corresponding to the second element of the list have the 
following columns:

* <div style="text-align: justify"> **ID** - A code identifying the GFAG (GO
term, KEGG pathway or one according to users annotations).</div> 
* <div style="text-align: justify"> **Description** - Description of the 
GFAG.</div> 
* <div style="text-align: justify"> **Raw_Number_Genes** -  Total number of 
genes related to the GFAG.</div>
* <div style="text-align: justify"> **Sample_Number_Genes** - Number of genes, 
present in the sample, related to the GFAG. </div> 
* <div style="text-align: justify"> **H_** - Two columns. GFAG gene diversity
of each sample (control *versus* experiment).</div> 
* <div style="text-align: justify"> **N_** - Two columns. GFAG gene activity 
of each sample (control *versus* experiment).</div> 
* <div style="text-align: justify"> **h** - Relative gene diversity. </div> 
* <div style="text-align: justify"> **n** - Relative gene activity. </div> 
* <div style="text-align: justify"> **pValue_h** - GFAG p-value related to 
gene diversity.</div> 
* <div style="text-align: justify"> **pValue_n** - GFAG p-value related to 
gene activity.</div> 
* <div style="text-align: justify"> **qValue_h** - GFAG corrected p-value 
related to gene diversity.</div> 
* <div style="text-align: justify"> **qValue_n** - GFAG corrected p-value 
related to gene activity.</div> 
* <div style="text-align: justify"> **Significance_h** - GFAG significance
related to gene diversity. "significative" means the q-value is lower than
the cutoff set by PCorrection argument, while "not-significative" means the
opposite.</div> 
* <div style="text-align: justify"> **Significance_n** - GFAG significance
related to gene activity. "significative" means the q-value is lower than
the cutoff set by PCorrection argument, while "not-significative" means the
opposite.</div> 
* <div style="text-align: justify"> **Wilcox_pvalue** - GFAG p-value generated
by Wilcoxon rank test.</div>
* <div style="text-align: justify"> **Wilcox_qvalue** - Wilcoxon GFAG 
corrected p-value.</div>
* <div style="text-align: justify"> **Wilcox_significance** - GFAG 
significance related Wilcoxon test. "significative" means the q-value is lower
than the cutoff set by PCorrection argument, while "not-significative" means 
the opposite.</div> 
* <div style="text-align: justify"> **Fisher_pvalue** - GFAG p-value generated
by Fisher's exact test.</div>
* <div style="text-align: justify"> **Fisher_qvalue** - Fisher GFAG corrected
p-value.</div>
* <div style="text-align: justify"> **Fisher_significance** - GFAG 
significance related to Fisher's exact test. "significative" means the q-value
is lower than the cutoff set by PCorrection argument, while 
"not-significative" means the opposite.</div> 

# ADAnalysis
*ADAnalysis* function provides a partial analysis, where is calculated just
gene diversity and activity of each GFAG with no signicance by bootstrap, 
Wilcoxon or Fisher. As an example, lets consider the same gene expression set
of *Aedes aegypti* previously used in *GFAGAnalysis* funcion example:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
suppressMessages(library(ADAM))
data("ExpressionAedes")
data("KeggPathwaysAedes")
```

As ADAM always need two samples (control *versus* experiment), let's select 
two sample columns from the expression data and define minimum and maximum 
number of genes per GFAG:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Comparison <- c("control1,experiment1")
Minimum <- 3
Maximum <- 100
```

*Aedes aegypti* doesn't have an annotation package. This way, we build our own
file:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
SpeciesID <- "KeggPathwaysAedes"
```

It's necessary to inform which function domain and gene nomenclature are being 
used. *Aedes agypti* doesn't have an annotation package. So the domain will be 
"own" and the nomenclature "geneStableID":

```{r eval=TRUE, out.height.px=6, out.width.px=6}
Domain <- "own"
Nomenclature <- "geneStableID"
```

As all arguments were defined, then we can run ADAnalysis function:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
ResultAnalysis <- suppressMessages(ADAnalysis(ComparisonID = Comparison, 
                            ExpressionData = ExpressionAedes,
                            MinGene = Minimum,
                            MaxGene = Maximum,
                            DBSpecies = KeggPathwaysAedes, 
                            AnalysisDomain = Domain, 
                            GeneIdentifier = Nomenclature))
```

In the example above, we used the function *supressMessages* just to stop 
showing messages during the *ADAnalysis* function execution. The output of
*ADAnalysis* will be a *list* with two elements. The first corresponds to a
*data frame* showing genes and their respective functions:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
head(ResultAnalysis[[1]])
```

The second element of the output list result corresponds to data frames
according to the argument ComparisonID:

```{r eval=TRUE, out.height.px=6, out.width.px=6}
DT::datatable(as.data.frame(ResultAnalysis[[2]][1]), width = 800, 
            options = list(scrollX = TRUE))
```

The data frames corresponding to the second element of the list have the 
following columns:

* <div style="text-align: justify"> **ID** - A code identifying the GFAG (GO
term, KEGG pathway or one according to users annotations).</div> 
* <div style="text-align: justify"> **Description** - Description of the 
GFAG.</div> 
* <div style="text-align: justify"> **Raw_Number_Genes** - Total number of genes
related to the GFAG. </div>
* <div style="text-align: justify"> **Sample_Number_Genes** - Number of genes,
present in the sample, related to the GFAG. </div> 
* <div style="text-align: justify"> **H_** - Two columns. GFAG gene diversity
of each sample (control *versus* experiment).</div> 
* <div style="text-align: justify"> **N_** - Two columns. GFAG gene activity 
of each sample (control *versus* experiment).</div> 
* <div style="text-align: justify"> **h** - Relative gene diversity. </div> 
* <div style="text-align: justify"> **n** - Relative gene activity. </div> 


# Session information

```{r, eval=TRUE, out.height.px=6, out.width.px=6, label='Session information', , echo=FALSE}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{KeggPathwaysAedes}
\alias{KeggPathwaysAedes}
\title{Relation between Aedes aegypti genes and KEGG pathways as 
ADAM input}
\format{
A data frame with 200 rows and 2 variables
}
\description{
A relation between the genes in the ExpressionAedes data and
their respective KEGG pathways (GFAGs).
}
\examples{
data(KeggPathwaysAedes)
}
\references{
Molan, A. L. 2018. “Construction of a Tool for Multispecies
Genic Functional Enrichment Analysis Among Comparative Samples.”
Master’s thesis Institute of Biosciences of Botucatu – Univ. Estadual
Paulista. http://hdl.handle.net/11449/157105.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{ExpressionAedes}
\alias{ExpressionAedes}
\title{Aedes aegypti RNA-seq data expression}
\format{
A data frame with 34 rows and 5 variables
}
\description{
A sample fragment of gene expression from an RNA-seq 
experiment of Aedes aegypti mosquito.
}
\examples{
data(ExpressionAedes)
}
\references{
AKBARI, O. S. et al. The developmental transcriptome of the 
mosquito aedes aegypti, an invasive species and major arbovirus vector. G3:
Genes— Genomes— Genetics, Genetics Society of America, v. 3, n. 9, 
p. 1493–1509, 2013.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analysis.R
\name{GFAGAnalysis}
\alias{GFAGAnalysis}
\title{Group of Functionally Associated Genes (GFAG) complete analysis}
\usage{
GFAGAnalysis(ComparisonID, ExpressionData, MinGene, MaxGene, 
SeedNumber, BootstrapNumber, PCorrection, DBSpecies, PCorrectionMethod, 
WilcoxonTest, FisherTest, AnalysisDomain, GeneIdentifier)
}
\arguments{
\item{ComparisonID}{Sample comparisons identification. It must be a vector
in which each element corresponds to 2 sample columns from the expression 
data. The data's sample columns in each element from the vector are comma 
separated. The first one refers to the control sample, while the second
refers to the experiment. This argument must be informed by the user. There 
is no default value for it.}

\item{ExpressionData}{Gene expression data (microarray or RNA-seq, for
example). It must be a SummarizedExperiment object, a data frame or a path
for a text file tab separated containing at least 3 columns. First column 
mandatory corresponds to the gene identification, according to 
GeneIdentifier argument. Second, third and the other columns correspond to
the gene expression values realated to the genes in the first column and
each of these columns correspond to a different sample (control versus 
experiment). This argument must be informed by the user. There 
is no default value for it.}

\item{MinGene}{Minimum number of genes per GFAG. It must be a positive
integer value different from zero and lower than MaxGene argument.
Default is 3.}

\item{MaxGene}{Maximum number of genes per GFAG. It must be a positive
integer value different from zero and greater than MinGene argument.
Default is 2000.}

\item{SeedNumber}{Seed for bootstrap. A numeric positive value used as seed
for random number generating by bootstrap function. Default is 1049.}

\item{BootstrapNumber}{Number of bootstraps. A numeric value greater than 
zero, used by bootstrap function generates p-values for each GFAG. Default
is 1000.}

\item{PCorrection}{Cutoff for p-value correction. A numeric value between 
0 and 1. Default is 0.05.}

\item{DBSpecies}{A string corresponding to the name of an OrgDb species 
gene annotation package: org.Ag.eg.db (Anopheles gambiae), org.At.tair.db 
(Arabdopsis thaliana), org.Bt.eg.db (Bos taurus), org.Ce.eg.db
(Caenorhabditis elegans), org.Cf.eg.db (Canis familiaris), org.Dm.eg.db 
(Drosophila melanogaster), org.Dr.eg.db (Danio rerio), org.EcK12.eg.db 
(Escherichia coli K12), org.EcSakai.eg.db (Escherichia coli Sakai),
org.Gg.eg.db (Gallus gallus), org.Hs.eg.db (Homo sapiens), org.Mm.eg.db
(Mus musculus), org.Mmu.eg.db (Macaca mulatta), org.Pf.plasmo.db
(Plasmodium falciparum), org.Pt.eg.db (Pan troglodytes), org.Rn.eg.db
(Rattus norvegicus), org.Sc.sgd.db (Saccharomyces cerevisiae),
org.Ss.eg.db (Sus scrofa) and org.Xl.eg.db (Xenopus laevis).
If there is no package, it's possible for the user to create a personal 
gene annotation file, tab separated, containing 3 columns: gene, term 
annotation code and description of the term annotation. So, istead of a 
string with an OrgDb name, inform a data frame or a path for the file.
This argument must be informed by the user. There is no default value for
it.}

\item{PCorrectionMethod}{Method p-value correction: holm, hochberg, hommel,
bonferroni, bh, by or fdr. Default is "fdr".}

\item{WilcoxonTest}{A logical value indicating whether or not to perform
Wilcoxon test (TRUE for performing and FALSE for not performing).
Default is FALSE.}

\item{FisherTest}{A logical value indicating whether or not to perform
Fisher's exact test (TRUE for performing and FALSE for not performing).
Default is FALSE.}

\item{AnalysisDomain}{Analysis domain to be considered for building GFAGs, 
according: gobp (Gene Ontology - Biological Processes), gocc (Gene Ontology
 - Celular Components), gomf (Gene Ontology - Molecular Functions), kegg
(KEGG Pathways) or own (if there is no annotation package - the annotations 
were defined in a file by the user). This argument must be informed by the 
user. There is no default value for it.}

\item{GeneIdentifier}{Gene nomenclature to be used: entrez or tairID
for Arabdopsis thaliana, entrez or orfID for Saccharomyces cerevisiae, 
symbol or orfID for Plasmodium falciparum and symbol or entrez for the 
other species. If there is no annotation package, just put the 
gene nomenclature present in the user's personal annotations. This argument 
must be informed by the user. There is no default value for it.}
}
\value{
Return a list with two elements. The first one refers to a 
data frame with the GFAGs and their respective genes. The second one is a 
a list where each position is a data frame presenting the result of the 
analysis, according to ComparisonID argument.
}
\description{
Analysis of functionally associated gene groups, based on
gene diversity and activity, for different species according to existing 
annotation packages or user's personal annotations. GFAGAnalysis function 
allows to run a complete analysis, using all available arguments.
}
\details{
The genes present in the expression data are grouped by their 
respective functions according to the domains described by 
AnalysisDomain argument. The relationship between genes and functions are 
made based on the species annotation package. If there is no annotation 
package, a three column file (gene, function and function description) must
be provided. For each GFAG, gene diversity and activity in each sample are 
calculated. As the package always compare two samples (control versus 
experiment), relative gene diversity and activity for each GFAG are
calculated. Using bootstrap method, for each GFAG, according to relative 
gene diversity and activity, two p-values are calculated. The p-values are 
then corrected, according to the correction method defined by 
PCorrectionMethod argument, generating a q-value. The significative GFAGs 
will be those whoose q-value stay under the cutoff set by PCorrection 
argument. Optionally, it's possible to run Wilcoxon test and/or Fisher's
exact test. These tests also provide a corrected p-value, 
and siginificative groups can be seen through them.
}
\examples{
##
#Complete Analysis with Aedes aetypti through GFAGAnalysis function
##
data(ExpressionAedes)
data(KeggPathwaysAedes)
ResultAnalysis <- GFAGAnalysis(ComparisonID = c("control1,experiment1", 
"control2,experiment2"), ExpressionData = ExpressionAedes, MinGene = 3L,
MaxGene = 20L, SeedNumber = 1049, BootstrapNumber = 1000L,
PCorrection = 0.05, DBSpecies = KeggPathwaysAedes,
PCorrectionMethod = "fdr", WilcoxonTest = TRUE, FisherTest = TRUE,
AnalysisDomain = "own", GeneIdentifier = "gene")
\dontrun{
head(ResultAnalysis[[1]]) #Relation between genes and functions
head(ResultAnalysis[[2]][1]) #Result comparison 1
head(ResultAnalysis[[2]][2]) #Result comparison 2
}
}
\references{
CASTRO, M. A., RYBARCZYK-FILHO, J. L., et al. Viacomplex:
software for landscape analysis of gene expression networks in genomic
context. Bioinformatics, Oxford Univ
Press, v. 25, n. 11, p. 1468–1469, 2009.
}
\author{
André Luiz Molan (andre.molan@unesp.br)
}
